from PyQt4.QtCore import *
from PyQt4.QtGui import *
from qgis.core import *
from qgis.utils import *
import pdb

import resources
from QBEform import QBEform

class QueryByExample:

    def __init__(self, iface):
        self.iface = iface
        self.layerids = []
        self.user_plugin_dir = QFileInfo(QgsApplication.qgisUserDbFilePath()).path() + "/python/plugins"
        self.dir = self.user_plugin_dir + "/QueryByExample/"

    def initGui(self):
        self.action = QAction(QIcon(":/plugins/QueryByExample/icon.png"),
                              "Query By Example",
                              self.iface.mainWindow());
        QObject.connect(self.action, SIGNAL("triggered()"), self.run)

        # Add toolbar button and menu item
        self.iface.addToolBarIcon(self.action)
        self.iface.addPluginToMenu("&QBE", self.action)
    
    def unload(self):
        # Remove the plugin menu item and icon
        self.iface.removePluginMenu("&QBE", self.action)
        self.iface.removeToolBarIcon(self.action)
        
    def run(self):
        self.dlg = QBEform()


        self.mapInstance = QgsMapLayerRegistry.instance()
        self.mapLayers = self.mapInstance.mapLayers()
        if (len(self.mapLayers) < 2):
            QMessageBox.information(self.iface.mainWindow(),
                                    "Error",
                                    "Need at least two layers")
            return
        for (name, layer) in self.mapLayers.iteritems():
            if layer.type() == QgsMapLayer.VectorLayer:
                tempname = layer.name()
                self.layerids.append(layer.id())
                self.dlg.ui.comboBoxWhere.addItem(tempname)
                self.dlg.ui.comboBoxTo.addItem(tempname)
        
        QObject.connect(self.dlg.ui.pushButtonApply,
                        SIGNAL("clicked()"),
                        self.querybe)

        self.dlg.show()

    def querybe(self):
        whereLayer = QgsVectorLayer
        toLayer = QgsVectorLayer
        try:
            whereIndex = self.dlg.ui.comboBoxWhere.currentIndex()
            toIndex = self.dlg.ui.comboBoxTo.currentIndex()
            print toIndex
            if (whereIndex == toIndex):
                QMessageBox.warning(self.iface.mainWindow(),
                                    "Error",
                                    "Two layers cannot be the same layer")
                return

            whereStr = self.layerids[whereIndex]
            whereLayer = self.mapInstance.mapLayer(whereStr)


            toStr = self.layerids[toIndex]
            print toStr
            toLayer = self.mapInstance.mapLayer(toStr)

            print toLayer
        except:
            QMessageBox.information(self.iface.mainWindow(),
                                    "Error",
                                    toStr + "is not a valid vector layer")
            return

        whereProvider = whereLayer.dataProvider()
        toProvider = toLayer.dataProvider()
        self.dlg.ui.progressBar.reset()
        self.totalFeatureCount = whereProvider.featureCount()
        self.currentFeatureCount = 0


        relationIndex = self.dlg.ui.comboBoxRelation.currentIndex()
        relationOps = {0 : [self.existsOne, self.judgeIntersects],
                       1 : [self.existsOne, self.judgeContains],
                       2 : [self.everyNot, self.judgeDisjoint],
                       3 : [self.existsOne, self.judgeEquals],
                       4 : [self.existsOne, self.judgeTouches],
                       5 : [self.existsOne, self.judgeOverlaps],
                       6 : [self.existsOne, self.judgeWithin],
                       7 : [self.existsOne, self.judgeCrosses]
                       }[relationIndex]
        selection = relationOps[0](whereProvider, toProvider, relationOps[1])

        whereLayer.setSelectedFeatures(selection)

        self.dlg.ui.labelInfo.setText(str(len(selection)) +
                                      "  feature(s) in selection")
        self.dlg.ui.progressBar.reset()
        
    def existsOne(self, whereProvider, toProvider, relationOp):
        whereProvider.select(whereProvider.attributeIndexes() )
        whereProvider.rewind()

        whereFeature = QgsFeature()
        toFeature = QgsFeature()
        selection = []        

        while (whereProvider.nextFeature(whereFeature)):
            whereGeom = whereFeature.geometry()
            toProvider.select(toProvider.attributeIndexes() )
            toProvider.rewind()

            while(toProvider.nextFeature(toFeature)):
                toGeom = toFeature.geometry()

                if (relationOp(whereGeom, toGeom) ):
                    selection.append(whereFeature.id())
                    break

            self.currentFeatureCount += 1
            self.dlg.ui.progressBar.setValue(int (float(self.currentFeatureCount) /
                                                  self.totalFeatureCount * 100))


        return selection

    def everyNot(self, whereProvider, toProvider, relationOp):
        whereProvider.select(whereProvider.attributeIndexes() )
        whereProvider.rewind()

        whereFeature = QgsFeature()
        toFeature = QgsFeature()
        selection = []        

        while (whereProvider.nextFeature(whereFeature)):
            whereGeom = whereFeature.geometry()
            toProvider.select(toProvider.attributeIndexes() )
            toProvider.rewind()

            exception = False
            while(toProvider.nextFeature(toFeature)):
                toGeom = toFeature.geometry()

                exception = False
                if (relationOp(whereGeom, toGeom) ):
                    pass
                else:
                    exception = True
                    break
            if (not exception):
                selection.append(whereFeature.id())

            self.currentFeatureCount += 1
            self.dlg.ui.progressBar.setValue(int (float(self.currentFeatureCount) /
                                                  self.totalFeatureCount * 100))
            
                
        return selection


    def judgeIntersects(self, where, to):
        return where.intersects(to)
    def judgeContains(self, where, to):
        return where.contains(to)
    def judgeDisjoint(self, where, to):
        return where.disjoint(to)
    def judgeEquals(self, where, to):
        return where.equals(to)
    def judgeTouches(self, where, to):
        return where.touches(to)
    def judgeOverlaps(self, where, to):
        return where.overlaps(to)
    def judgeWithin(self, where, to):
        return where.within(to)
    def judgeCrosses(self, where, to):
        return where.crosses(to)
                      
